/*
*   This file is part of the Interactive Julia and Mandelbrot system.
*   The Interactive Julia and Mandelbrot system is free software: 
*	you can redistribute it and/or modify it under the terms of the 
*	GNU General Public License as published by the Free Software 
*	Foundation, either version 3 of the License, or (at your option) 
*	any later version.
*
*   The Interactive Julia and Mandelbrot system is distributed in 
*	the hope that it will be useful, but WITHOUT ANY WARRANTY; 
*	without even the implied warranty of MERCHANTABILITY or FITNESS 
*	FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
*	for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with The Interactive Julia and Mandelbrot suite.  
*	If not, see <http://www.gnu.org/licenses/>.
*/

import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
/**
 * The JuliaFrame class allows the user to run an animation showing how the Julia
 * sets change as the value of C changes by moving along a path set by the user 
 * by mouse dragging on the image of the Mandelbrot set which is displayed on another
 * frame, which sends information to the instance of this class.
 */
class JuliaFrame extends JFrame {
	MandelbrotFrame mandelbrotCopy; // Global variable used to reference the JFrame holding the Mandelbrot image
	double[] xArray; // Array holding x (real) components of animation values of c
	double[] yArray; // Array holding y (imaginary) components of animation values of c
	int topIndex; // top index of arrays xArray and yArray
	private static final int H = 301; // Height of window
	private static final int W = 401; // Width of window
	private Color previousColour = new Color(00, 00, 00);
	private Color currentColour = new Color(00, 00, 00);
	private double firstChangex = 0; // Used when drawing the Julia image, holds the position on line where current colour starts
	private final double minXDefault = -2; // default lower bound of x-range plotted
	private final double maxXDefault = 2;  // default upper bound of x-range plotted
	private final double minYDefault = -1.5; // default lower bound of y-range plotted
	private final double maxYDefault = 1.5; // default upper bound of y-range plotted
	private double minX = minXDefault; // lower bound of x-range plotted, may be changed by user zooming in
	private double maxX = maxXDefault; // upper bound of x-range plotted, may be changed by user zooming in
	private double minY = minYDefault; // lower bound of y-range plotted, may be changed by user zooming in
	private double maxY = maxYDefault; // upper bound of y-range plotted, may be changed by user zooming in
	int maxIterations = 300; // maximum iterations for each complex number, if it hasn't diverged it is assumed it won't
	private int xMargin = 150; // gap between the left side of JFrame and the plotted image
	private int yMargin = 40; // gap between lower edge of plotted image and bottom of the JFrame
	private int topOffset = 54; // gap between top of JFrame and the plotted image
	private int rightMargin = 30; // gap between right edge of plotted image and the edge of the JFrame
	private int yAxisGap = 8; // gap between y-axis and plotted image
	private int xAxisGap = 8; // gap between x-axis and plotted image
	private double imC = 0; // imaginary component of the current value of complex number c
	private double reC = 0; // real component of the current value of complex number c  
	private int index = 0;
	private Transaction theAL = new Transaction(); // Used to implement the buttons' actions
	boolean animationStart = false; // indicates if the animation is running
	boolean firstDisplay = true; // Has a Julia set been displayed yet ?
	int delay = 120; // default pause between frames
	boolean forward = true; // direction of animation
	private BufferedImage theAI; // the image that holds the plot displayed on the frame 
	private Graphics2D theAG; // graphics for the image
	private BufferedImage theAIToSave; // the image that holds the plot to be saved to disk 
	private Graphics2D theAGToSave; // graphics for the image to be saved
	private Color[] juliaColours = {new Color(36, 159, 120), new Color(0,50,150), new Color(36, 159, 120),
			new Color(80, 180, 80), new Color(140, 50, 180), // colours in which to plot the divergent points         
			new Color(130, 50, 50), new Color(0, 160, 160), 
			new Color(190, 160, 160), new Color(0, 255, 255), 
			new Color(125, 220, 140), new Color(190,220,100),
			new Color(230,230,50)};
	private int[] juliaCutOffs = {2,3,4,5,7,9,11,14, 17, 25, 50, 58};  // cut-offs for the number of iterations that trigger a change of colour

	private Iterator iterator;  // will reference an instance of Iterator which returns a colour for each pixel
	private AxisWiz axisWiz; // Will hold an instance for drawing the axes and setting the axis labels' values

	private int mouseX; // x coordinate of mouse position
	private int mouseY; // y coordinate of mouse position

	private JButton c = new JButton("Forward");
	private JButton d = new JButton("Back");
	private JMenuBar menubar = new JMenuBar();
	private JMenu file;
	private JMenu zoom;
	private JMenu axes;
	private JMenuItem save;
	private JMenuItem exit;
	private JMenuItem setRanges;
	private JMenuItem normal;
	private JMenuItem show;
	private JMenuItem hide;
	private boolean showAxis = false; // True id axes are visible
	private JLabel[] axisLabel = {new JLabel(String.valueOf(maxY)), new JLabel(String.valueOf((maxY + minY)/2)), 
			new JLabel(String.valueOf(minY)), new JLabel(String.valueOf(minX)),
			new JLabel(String.valueOf((maxX + minX)/2)), new JLabel(String.valueOf(maxX))};  
	String outputC; // for writing the current value of C on the plot
	private boolean zoomSelect = false; // True if the user has selected to zoom in
	JOptionPane frame = new JOptionPane();
	DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss"); // format for putting date in name of saved image
	MyJCanvas canvas = new MyJCanvas();  // A subclass of JPanel on which the graphic image is drawn
	Color backgroundColour =  new Color(240, 240, 240);
	private boolean animationEnding = false;

	/**
	 * The constructor adds the menu bar and drop down menus, and sets
	 * the location, size, etc of the frame. It creates the instances of Iterator 
	 * and Axiswiz that will be used by this instance of JuliaFrame.The constructor
	 * also receives as parameter the instance of MandelbrotFrame with which this
	 * frame will interact, and gives it to the global variable mandelbrotCopy. It 
	 * also creates the instance of Animate which creates the thread which runs the 
	 * animation. The constructor also places buttons on the frame.
	 * @param mandelbrot - MandelbrotFrame
	 */	
	public JuliaFrame(MandelbrotFrame mandelbrot) {

		mandelbrotCopy = mandelbrot;		
		setLocation(750, 0);
		setSize(W + xMargin + rightMargin, H + yMargin + topOffset ); // Size of drawing area
		setTitle("Julia set animation");
		setResizable(false);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		Animate animation = new Animate();
		Container cp = getContentPane();
		cp.setLayout(null);
		
		canvas.setBounds(xMargin - xAxisGap - 8, topOffset - 48, W + xAxisGap + 9, H + yAxisGap + 8);
		canvas.setBackground(backgroundColour);
		cp.add(canvas);
		
		JButton a = new JButton("Start");
		a.setBounds(10, 1 * 40 - 10, 95, 30);
		a.addActionListener(theAL);
		cp.add(a);

		JButton b = new JButton("Stop");
		b.setBounds(10, 2 * 40 - 10, 95, 30);
		b.addActionListener(theAL);
		cp.add(b);

		c.setBounds(10, 3 * 40 - 10, 95, 30);
		c.addActionListener(theAL);
		c.setEnabled(false);
		cp.add(c);

		d.setBounds(10, 4 * 40 + 20, 95, 30);
		d.addActionListener(theAL);
		d.setEnabled(false);
		cp.add(d);

		JButton e = new JButton("Faster");
		e.setBounds(10, 5 * 40 + 20, 95, 30);
		e.addActionListener(theAL);
		cp.add(e);

		JButton f = new JButton("Slower");
		f.setBounds(10, 6 * 40 + 20, 95, 30);
		f.addActionListener(theAL);
		cp.add(f);

		iterator = new Iterator(maxIterations, juliaColours, juliaCutOffs);
		axisWiz = new AxisWiz(W, H);

		axisWiz.placeAxisLables(this, axisLabel, xMargin, topOffset);

		axisWiz.setAxisLabels(axisLabel, maxY, (maxY + minY)/2, 
				minY, minX, (maxX + minX)/2, maxX);

		menubar = new JMenuBar();
		setJMenuBar(menubar);

		file = new JMenu("File");
		zoom = new JMenu("Zoom");
		axes = new JMenu("Axes");
		
		file.setEnabled(false);
		zoom.setEnabled(false);

		menubar.add(file);
		menubar.add(zoom);
		menubar.add(axes);

		save = new JMenuItem("Save");
		exit = new JMenuItem("Exit");
		setRanges = new JMenuItem("Select Area With Mouse (on/off)");
		normal = new JMenuItem("Normal");
		show = new JMenuItem("Show");
		hide = new JMenuItem("Hide");

		file.add(save);
		file.add(exit);
		zoom.add(setRanges);
		zoom.add(normal);
		axes.add(show);
		axes.add(hide);

		exit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				System.exit(0);
			}
		});

		save.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				theAIToSave = (BufferedImage) createImage(W + 130, H + 100);
				theAGToSave  = theAIToSave.createGraphics();
				theAGToSave.drawImage(theAI, 80, 50, null);
				if(showAxis) 
				{
					axisWiz.drawAxis(theAGToSave, 80, 50, 8, 8);
					theAGToSave.drawString(axisWiz.axisLabelNumber(maxY), 0, 53);
					theAGToSave.drawString(axisWiz.axisLabelNumber((minY + maxY)/2),0, 53 + H/2);
					theAGToSave.drawString(axisWiz.axisLabelNumber(maxY),0, 53 + H);
					theAGToSave.drawString(axisWiz.axisLabelNumber(minX), 47, 50 + H + 32);
					theAGToSave.drawString(axisWiz.axisLabelNumber((minX + maxX)/2), 47 + W/2, 50 + H + 32);
					theAGToSave.drawString(axisWiz.axisLabelNumber(maxX), 47 + W, 50 + H + 32);
				}

				try {
					ImageIO.write(theAIToSave, "png", new File("JuliaImage " + processDate(new Date(), dateFormat) + ".png"));
				} catch (IOException e) {
					e.printStackTrace();
				}
				repaint();
			}
		});


		show.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				showAxis = true;
				axisWiz.setVisibility(axisLabel, true);
				repaint();
			}
		});

		hide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				showAxis = false;
				axisWiz.setVisibility(axisLabel, false);
				repaint();
			}
		});  	  

		normal.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				minX = minXDefault;
				maxX = maxXDefault;
				minY = minYDefault;
				maxY = maxYDefault;
				axisWiz.setAxisLabels(axisLabel, maxY, (maxY + minY)/2, 
						minY, minX, (maxX + minX)/2, maxX);
				background();
				repaint();
			}
		}); 

		setRanges.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (!firstDisplay) 
				{
					zoomSelect = !zoomSelect; //toggle
					if(zoomSelect)
					{
						canvas.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
					}
					else
					{
						canvas.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					}
					repaint();
				}
			}
		});
		setVisible(true);
	} // end of constructor

	/**
	 * This method receives the arrays holding the animation pathway (i.e. the successive
	 * values of C from the Mandelbrot frame, along with the highest index position, and it 
	 * initialises the array access variable index to zero.
	 * @param xArrayIn -  double[]
	 * @param yArrayIn - double[]
	 * @param arrayMax - int
	 */
	public void pathSet(double[] xArrayIn, double[] yArrayIn, int arrayMax )
	{
		xArray = xArrayIn;
		yArray = yArrayIn;
		topIndex = arrayMax;
		index = 0;
	}
/**
 * 
 * The class transaction provides the action listener for the buttons
 *
 */
	class Transaction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			String actionIs = ae.getActionCommand();

			if (actionIs.equals("Start"))
			{ 
				if(topIndex < 5)
				{
					JOptionPane.showMessageDialog(frame, "No Path Set, Please set path by dragging on the Mandelbrot set");
				}
				else
				{
					animationStart = true;	
					mandelbrotCopy.applicationRunning(true);
					c.setEnabled(true);
					d.setEnabled(true);
					file.setEnabled(true);
					zoom.setEnabled(true);	

				}    	  		    	    
			} 
			if (actionIs.equals("Stop"))
			{
				animationStart = false;
				animationEnding = true;
				c.setEnabled(false);
				d.setEnabled(false);
			}
			if (actionIs.equals("Forward"))
			{
				forward = true;
			}
			if (actionIs.equals("Back"))
			{
				forward = false;
			}
			if (actionIs.equals("Faster"))
			{
				if (delay > 30)
					delay -= 10;
			}
			if (actionIs.equals("Slower"))  
			{
				delay += 10;
			}	  
		}
	}
/**
 * 
 * The class Animate provides the thread that runs the animation
 *
 */
	class Animate implements Runnable
	{

		private Thread theAnimation;
		/**
		 * The constructor creates the thread and starts it running
		 */
		public Animate()
		{
			theAnimation = new Thread(this);
			theAnimation.start();
		}
        /**
         * SetC() sets accesses the next value of C from the arrays holding the pathway
         * set by the user. Each value of C is used to draw a Julia set which will be the
         * next  frame in the animation.
         */
		public synchronized void setC()
		{	
			reC = xArray[index];
			imC = yArray[index];
		}
        /**
         * The run method of the thread driving the animation gets the value of C for the next plot,
         * then gets the next plot drawn, sleeps for 10 millisecs redisplays the graphics and tells
         * the Mandelbrot frame to update itself. The thread sleeps for a short time set by the 
         * variable delay.
         */
		public void run() {
			while( true )
			{
				try
				{

					if (animationStart)
					{
						setC(); // get value of C for next plot
						background(); // plot next Julia set
						canvas.repaint(); // Update the panel holding the graphics
						mandelbrotCopy.traceC(index); // Tell the Mandelbrot frame to update the crosshairs
						if(forward)	index++;
						else index--;
						if(index > topIndex) index = 0;
						if(index < 0) index = topIndex;
						firstDisplay = false;
					}

					Thread.sleep(delay);	
					if(animationEnding)
					{
						mandelbrotCopy.applicationRunning(false);
						animationEnding = false;	
					}
									
				}
				catch (InterruptedException e) {}
			}	
		}	
	}

	public void update(Graphics g)
	{
		paint((Graphics2D) g);
	}

	/**
     * The paint() method invokes the method of JFrame and displays the axes if required
     * @param g - Graphics
     */
	public void paint(Graphics g) // When 'Window' is first
	{ 							  // shown or damaged
		super.paint(g);
	}
	/**
	 * Draws the next Julia set, and writes on it the value of C
	 */
	public void background()
	{
		theAI = (BufferedImage) createImage(W, H);
		theAG = theAI.createGraphics();	
		reDrawJulia(theAG);
		outputC = "C = " + ((double) Math.round( reC*1000000))/1000000 + " + " + ((double) Math.round( imC*1000000))/1000000 + "i";	
		theAG.setPaint(new Color(255, 255, 255));
		theAG.drawString(outputC, 10,  290); 		
	}
	/**
	 * Turns the date and time into a string that is used in the name of the saved image
	 * @param inputDate -  Date
	 * @param format - DateFormat
	 * @return dateString -  string
	 */
	public String processDate(Date inputDate, DateFormat format) 
	{ // formats date and time for output in text file
		String dateString = format.format(inputDate);
		dateString = dateString.replace(':', ' ');
		dateString = dateString.replace('/', ' ');
		return dateString;
	}
	/**
	 * This method redraws the Julia set by invoking the Iterator instance
	 * to return colour based on the number of iterations required for divergence
	 * or black for non-divergence. Each pixel represents a complex number which is
	 * the initial value of Z for the iteration of Z := Z^2 + C.
	 * @param g -  Graphics2D
	 */
	public void reDrawJulia(Graphics2D g) // Re draw contents
	{
		double x, y; // The mathematical x and y values corresponding to image pixels
		for(double yPixel = 0; yPixel < H; yPixel++)
		{
			for(double xPixel = 0; xPixel < W; xPixel++)
			{
				x = (minX)+(xPixel*((maxX - minX)/W));
				y = (maxY)-(yPixel*((maxY - minY)/H));
				previousColour = currentColour;		
				currentColour = iterator.colourPicker(reC, imC, x, y);
				if(! previousColour.equals(currentColour) || xPixel == W - 1)
				{
					g.draw(new Line2D.Double(firstChangex, yPixel, xPixel, yPixel));
					firstChangex = xPixel % (W - 1);
					g.setPaint(currentColour);
				}				
			}
		}
	}
    /**
     * @return - the real component of the current value of C used to plot the Julia set
     */
	public double getRe()
	{
		return reC;
	}
	/**
	 * @return - the imaginary component of the current value of C used to plot the Julia set
	 */
	public double getIm() 
	{
		return imC;
	}
    /**
     * This subclass of JPanel will have the images of the Julia sets 
     * drawn on it
     */
	public class MyJCanvas extends JPanel   {
		private int mouseXZoom;
		private int mouseYZoom;
		private double xZoom;
		private double yZoom;
		private int xDrag;
		private int yDrag;
		private boolean dragging = false;
		private double xPressed;
		private double yPressed; 
		String outputC;
        /**
         * The constructor adds the mouse listeners for zooming in on a selected part
         * of the plot
         */
		public MyJCanvas()
		{
			addMouseListener(new MouseAdapter() 		
			{
				public void mousePressed (MouseEvent e)
				{	    

					mouseX = e.getX();
					mouseY = e.getY();
					if (mouseInRange(mouseX, mouseY))
					{
						xPressed = ((minX)+((mouseX - xAxisGap - 8)*((maxX - minX)/W)));
						yPressed = ((maxY)-((mouseY)*((maxY - minY)/H)));
					}
					dragging = false;
				}
				public void mouseReleased (MouseEvent e)
				{
					if(zoomSelect && mouseInRange(e.getX(), e.getY()))
					{
						mouseXZoom = e.getX();
						mouseYZoom = e.getY();
						xZoom = (minX)+((mouseXZoom - 8 - xAxisGap)*((maxX - minX)/W));
						yZoom = (maxY)-((mouseYZoom)*((maxY - minY)/H));

						minX = Math.min(xPressed, xZoom);
						maxX = Math.max(xPressed, xZoom);
						minY = Math.min(yPressed, yZoom);
						maxY = Math.max(yPressed  , yZoom);

						axisWiz.setAxisLabels(axisLabel, maxY, (maxY + minY)/2, 
								minY, minX, (maxX + minX)/2, maxX);
						background();
					}
					zoomSelect = false;
					setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					dragging = false;   
					repaint();
				}
			});

			addMouseMotionListener(new MouseMotionAdapter() 
			{
				public void mouseDragged (MouseEvent e)
				{
					xDrag = e.getX();
					yDrag = e.getY();
					if (mouseInRange(xDrag, yDrag))
					{
						dragging = true;
						if (zoomSelect  && !animationStart) repaint();
					}
					else
					{
						dragging = false;
						zoomSelect = false;
						setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
						repaint();
					} 
				}
			});
		}
		/**
		 * The paintComponent (Graphics g) method repaints the panel with the next Julia plot and
		 * provides feedback when the mouse is used to select an area to zoom in on.
		 * @ param g - Graphics
		 */
		@Override
		public void paintComponent(Graphics g) {
            super.paintComponent(g);
            g.drawImage( theAI, xAxisGap + 8,  0,  this);
            if(firstDisplay) // initial display is black rectanle
    		{
    			((Graphics2D) g).setPaint(Color.BLACK);
    			((Graphics2D) g).fill( new Rectangle2D.Double( xAxisGap + 8, 0, W, H) );	
    		}
            if (zoomSelect && dragging) // visual feedback of zoom area being seleced
    		{
    			((Graphics2D) g).setPaint(Color.WHITE);
    			g.drawRect(Math.min(mouseX, xDrag), Math.min(mouseY, yDrag), Math.abs(mouseX - xDrag), Math.abs(mouseY - yDrag)); 
    		}
            if (showAxis) // display axes if required
    		{
    			axisWiz.drawAxis((Graphics2D) g, xAxisGap + 8, 0, xAxisGap, yAxisGap);
    		}
            if (zoomSelect) // visual feedback that zoom select is on
			{
				((Graphics2D) g).setPaint(new Color(255,50,50));  
				g.setFont(new Font("ARIAL", Font.BOLD, 14));
				g.drawString("Mouse drag over area to zoom in on",20, 20);
			}
        }
		/**
		 * A method to tell if the mouse is on the image of the Mandelbrot set
		 * @param mouseXp - int
		 * @param mouseYp - int
		 */
		public boolean mouseInRange(int mouseXp, int mouseYp)
		{
			return (mouseXp <= W + xAxisGap + 8 && mouseYp <= H && mouseXp >= xAxisGap + 8 && mouseYp >= 0);
		}
	}
	
}

