/*
*   This file is part of the Interactive Julia and Mandelbrot system.
*   The Interactive Julia and Mandelbrot system is free software: 
*	you can redistribute it and/or modify it under the terms of the 
*	GNU General Public License as published by the Free Software 
*	Foundation, either version 3 of the License, or (at your option) 
*	any later version.
*
*   The Interactive Julia and Mandelbrot system is distributed in 
*	the hope that it will be useful, but WITHOUT ANY WARRANTY; 
*	without even the implied warranty of MERCHANTABILITY or FITNESS 
*	FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
*	for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with The Interactive Julia and Mandelbrot suite.  
*	If not, see <http://www.gnu.org/licenses/>.
*/

import java.awt.AWTException;
/**
 * The MandelbrotFrame class is used to provides a frame for the graphic representation 
 * of the Mandelbrot set and provides for user interaction with it by mouse 
 * clicking or mouse dragging
 */
import java.util.ArrayList;
import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
//import java.util.Date;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.Line2D;
import java.awt.image.BufferedImage;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import java.io.File;
import java.io.IOException;
public class MandelbrotFrame extends JFrame {
	private static final int H = 540; // Height of window
	private static final int W = 630; // Width of window
	private Color previousColour = new Color(00, 00, 00);   // detects change of colour, so same colour is drawn as line
	private Color currentColour = new Color(00, 00, 00);    // colour returned by iterator for plot
	private Color backgroundColour = new Color(240, 240, 240); // background colour of frame
	private double firstChangex = 0; // used in plotting, holds x coordinate where colour starts
	private double minXDefault = -2.1; // minimum value of x in normal zoom state
	private double maxXDefault =  0.7; // maximum value of x in normal zoom state
	private double minYDefault = -1.2; // minimum value of y in normal zoom state
	private double maxYDefault = 1.2;  // maximum value of y in normal zoom state			
	private double minX = minXDefault; // initialise minimum x to default value
	private double maxX = maxXDefault; // initialise maximum x to default value
	private double minY = minYDefault; // initialise minimum y to default value
	private double maxY = maxYDefault; // initialise maximum x to default value
	private int maxIterations = 150; // maximum iterations for each complex number, if it hasn't diverged it is assumed it won't
	private int xMargin = 70; // gap between the left side of JFrame and the plotted image
	private int yMargin = 50; // gap between lower edge of plotted image and bottom of the JFrame
	private int topOffset = 130; // gap between top of JFrame and the plotted image
	private int rightMargin = 40; // gap between right edge of plotted image and the edge of the JFrame
	private final JTextField theRealOutput = new JTextField(); // text field for real component of C clocked on
	private final JTextField theImOutput = new JTextField();  // text field for imaginary component of C clocked on
	public double[] xArray, yArray; // hold x and y numerical values in animation path of values for C
	public int[] xPixelArray, yPixelArray; // hold x and y coordinates of pixels in animation path of values for C
	public int topArrayIndex = -1; // the highest index position in the arrays holing the path for the Julia animation.
	private Color[] mandelbrotColours = {new Color(36, 59, 120), new Color(52, 90, 180), new Color(158, 27, 224), // colours for plot
			new Color(45, 27, 156), new Color(0, 180, 180),new Color(146, 255, 100), new Color(255,255,255)};
	private int[] mandelbrotCutOffs = {1,7,13,19,30,70};  // cut-offs for the number of iterations that trigger a change of colour
	private BufferedImage theAI, theAIToSave;  // the images that hold the plot displayed on the frame and the plot to be saved to disk
	private Graphics2D theAG, theAGToSave; // graphics for the images
	private int animationIndex, animationX, animationY; // used by traceC() method to repaint area with crosshairs in it
	private Iterator iterator; // will reference an instance of Iterator which returns a colour for each pixel
	private JMenuBar menubar = new JMenuBar();
	private JMenu file, zoom, axes, path, help;
	private JMenuItem save, exit, setRanges, normal, show, hide, setStraightLine;
	private AxisWiz axisWiz; // Will hold an instance for drawing the axes and setting the axis labels' values
	private JLabel[] axisLabel = {new JLabel(String.valueOf(maxY)), new JLabel(String.valueOf((maxY + minY)/2)), 
			new JLabel(String.valueOf(minY)), new JLabel(String.valueOf(minX)),
			new JLabel(String.valueOf((maxX + minX)/2)), new JLabel(String.valueOf(maxX))};  
	private int xAxisGap = 8, yAxisGap = 8; // space between the axes (when visible) and the plotted image
	boolean zoomSelect = false; // True if the user has selected to zoom in
	private boolean showAxis = false; // True if the axes are visible
	private boolean animationRunning = false; // true if there is an animation running
	private JuliaFrame copyOfJulia; //global variable to access the Julia animation frame
	DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss"); // format for putting date in name of saved image
	MyCanvas canvas = new MyCanvas(); // A subclass of JPanel on which the graphic image is drawn
	MandelbrotFrame selfCopy = this;
	double[] straightPath;
	private boolean pathSet = false;
	private boolean settingPath = false, aCross = false;
	PathInputFrame f;
	private JButton p = new JButton("Plot  C");
	double Xpoint2 = 0, Ypoint2 = 0;;
	/**
	 * The constructor adds the menu bar and drop down menus, and sets
	 * the location, size, etc of the frame. It also adds the JLabels and 
	 * text boxes to the frame. It creates the instances of Iterator and
	 * Axiswiz that will be used by the instance of MandelbrotFrame. It also
	 * creates the instance of MyCanvas (a subclass of JPanel) that will have
	 * the graphics drawn on it, and positions it on the frame.
	 */	
	public MandelbrotFrame() {

		setSize(W + xMargin + rightMargin, H + yMargin + topOffset ); // Size of drawing area plus margins
		getContentPane().setBackground( backgroundColour);
		setDefaultCloseOperation(EXIT_ON_CLOSE);    
		setTitle("Mandelbrot set");
		setResizable(false); // Do not allow user to re-size because pixels represent mathematical points
		// Code to add widgets to the frame
		Container cp = getContentPane();
		cp.setLayout(null);
		theRealOutput.setBounds(190,45,80,30);
		cp.add(theRealOutput);	 
		theImOutput.setBounds(310,45,80,30);
		cp.add(theImOutput);
		setVisible(true);

		JLabel label1 = new JLabel("Click to set C  =");
		JLabel label2 = new JLabel(" + ");
		JLabel label3 = new JLabel("i");
		JLabel label4 = new JLabel("Mouse drag on the Mandelbrot image to set a path, then click Start button on the Julia animation");

		label1.setBounds(60,45,150,30);
		label1.setFont(new Font("ARIAL", Font.PLAIN, 16));
		cp.add(label1);

		label2.setBounds(280,45,50,30);
		label2.setFont(new Font("Serif", Font.PLAIN, 20));
		cp.add(label2);

		label3.setBounds(395,45,100,30);
		label3.setFont(new Font("Serif", Font.ITALIC, 20));
		cp.add(label3);

		label4.setBounds(60,5,800,30);
		label4.setFont(new Font("ARIAL", Font.PLAIN, 16));
		cp.add(label4);

		canvas.setBounds(xMargin - xAxisGap - 8, topOffset - 48, W + xAxisGap + 9, H + yAxisGap + 8 );
		canvas.setBackground(backgroundColour);
		cp.add(canvas);

		// Initialise the instance of Iterator and draw the Mandelbrot set as the background image
		iterator = new Iterator(maxIterations, mandelbrotColours, mandelbrotCutOffs);
		background();		

		axisWiz = new AxisWiz(W + 1, H + 1);

		axisWiz.placeAxisLables(this, axisLabel, xMargin, topOffset);

		axisWiz.setAxisLabels(axisLabel, maxY, (maxY + minY)/2, 
				minY, minX, (maxX + minX)/2, maxX);
		
		p.setBounds(445, 44 ,95, 28);
		cp.add(p);
		
		p.addActionListener(new ActionListener() {
			 
            public void actionPerformed(ActionEvent e)
            {
            	if(!animationRunning)
            	{
	            	boolean parse = true;
	            	try
					{
						Xpoint2 = Double.parseDouble(theRealOutput.getText());
						Ypoint2 = Double.parseDouble(theImOutput.getText());
					}
					catch(NumberFormatException f)
					{
						parse = false;
					}
	            	if(parse && Xpoint2 >= minX && Xpoint2 <= maxX && Ypoint2 >= minY && Ypoint2 <= maxY)
	            	{
	            		aCross = true;
	            		canvas.repaint();
	            	}
            	}
            	else
            	{
            		JOptionPane frame = new JOptionPane();
				    JOptionPane.showMessageDialog(frame, "Please stop the current animation before using the plot button");
            	}
            }
        });      

		menubar = new JMenuBar();
		setJMenuBar(menubar);

		file = new JMenu("File");
		zoom = new JMenu("Zoom");
		axes = new JMenu("Axes");
		path = new JMenu("Path");
		help = new JMenu("Help");

		menubar.add(file);
		menubar.add(zoom);
		menubar.add(axes);
		menubar.add(path);
		menubar.add(help);

		save = new JMenuItem("Save");
		exit = new JMenuItem("Exit");
		setRanges = new JMenuItem("Select Area With Mouse (on/off)");
		normal = new JMenuItem("Normal");
		show = new JMenuItem("Show");
		hide = new JMenuItem("Hide");
		setStraightLine = new JMenuItem("Set straight line path");
		
		JMenuItem about = new JMenuItem("about");
		JMenuItem instructions = new JMenuItem("instructions");

		file.add(save);
		file.add(exit);
		zoom.add(setRanges);
		zoom.add(normal);
		axes.add(show);
		axes.add(hide);
		path.add(setStraightLine);
		help.add(instructions);
		help.add(about);
		
		exit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				System.exit(0);
			}
		});
		/**
		 * transfers the image to TheAIToSAve, draws the axes if 
		 * they are visible and saves the image to disk, with the value
		 * of C plotted corresponding to the value of C in the Julia animation
		 */
		save.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				theAIToSave = (BufferedImage) createImage(W + 130, H + 130);
				theAGToSave  = theAIToSave.createGraphics();
				theAGToSave.drawImage(theAI, 80, 50, null);
				boolean parse = true;
				double Xpoint = 0;
				double Ypoint = 0;
				
				try
				{
					Xpoint = Double.parseDouble(theRealOutput.getText());
					Ypoint = Double.parseDouble(theImOutput.getText());
				}
				catch(NumberFormatException e)
				{
					parse = false;
				}
				
                if (Xpoint >= minX && Xpoint <= maxX && Ypoint >= minY && Ypoint <= maxY && parse)
                {
                    int xPix = (int) Math.round((Xpoint - minX)*(W/(maxX - minX))); 
				    int yPix = (int) Math.round((maxY - Ypoint)*(H/(maxY - minY)));
				    plotPoint(theAGToSave, 80 + xPix, 50 + yPix);
				    theAGToSave.setPaint(Color.BLACK);
				    theAGToSave.drawString("c = " + (double) Math.round(Xpoint*1000000)/1000000 + " + " + (double)  Math.round(Ypoint*1000000)/1000000 + " i", 20,  20);
                }
				if(showAxis) 
				{
					axisWiz.drawAxis(theAGToSave, 80, 50, 8, 8);
					theAGToSave.drawString(axisWiz.axisLabelNumber(maxY), 0, 53);
					theAGToSave.drawString(axisWiz.axisLabelNumber((minY + maxY)/2),0, 53 + H/2);
					theAGToSave.drawString(axisWiz.axisLabelNumber(maxY),0, 53 + H);
					theAGToSave.drawString(axisWiz.axisLabelNumber(minX), 47, 50 + H + 32);
					theAGToSave.drawString(axisWiz.axisLabelNumber((minX + maxX)/2), 47 + W/2, 50 + H + 32);
					theAGToSave.drawString(axisWiz.axisLabelNumber(maxX), 47 + W, 50 + H + 32);
				}

				try {
					ImageIO.write(theAIToSave, "png", new File("Mandelbrotimage" + processDate(new Date(), dateFormat) +".png"));
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			}
		});

		show.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				showAxis = true;
				axisWiz.setVisibility(axisLabel, true);
				repaint();
			}
		});

		hide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				showAxis = false;
				axisWiz.setVisibility(axisLabel, false);
				repaint();
			}
		});

		normal.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				minX = minXDefault;
				maxX = maxXDefault;
				minY = minYDefault;
				maxY = maxYDefault;
				axisWiz.setAxisLabels(axisLabel, maxY, (maxY + minY)/2, 
						minY, minX, (maxX + minX)/2, maxX);
				background();		
				if (pathSet)
					for(int i = 0; i <= topArrayIndex; i++)
					{
						xPixelArray[i] = xAxisGap + 8 + (int) Math.round((xArray[i] - minX)*(W/(maxX - minX)));
						yPixelArray[i] = (int) Math.round((maxY - yArray[i])*(H/(maxY - minY)));	
					}

				repaint();
			} 	
		}); 

		setRanges.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				zoomSelect = !zoomSelect; //toggle
				if(zoomSelect)
				{
					canvas.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
				}
				else
				{
					canvas.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				}
				canvas.repaint();
			}
		}); 

		setStraightLine.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(!settingPath) 
				{
					f = new PathInputFrame(selfCopy);
					settingPath = true;
				}
				else
				{
					f.setState(java.awt.Frame.NORMAL);
					f.toFront();
				}
			}	
		}); 
		
		about.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				String license = "Copyright 2014 Ian Kane\n\nThis program is free software: you can redistribute it and/or modify\n it under the terms of the GNU General Public License as published by\n" +
						"the Free Software Foundation, either version 3 of the License,\n or (at your option) any later version.\n\n This program is distributed in the hope that it will be useful,\n" +
						" but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY\n or FITNESS FOR A PARTICULAR PURPOSE.\n See the GNU General Public License for more details.\n\n" +
						" You should have received a copy of the GNU General Public License along with this program.\n If not, see <http://www.gnu.org/licenses/>.";
				JOptionPane frame = new JOptionPane();
				JOptionPane.showMessageDialog(frame, license);
			}
		});
		
		instructions.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				String license = "A set of instructions for this program can be found in the\nreadme file in the JuliaMandelbrot folder or zip archive";
				JOptionPane frame = new JOptionPane();
				JOptionPane.showMessageDialog(frame, license);
			}
		});

		setVisible(true);
		repaint();

	} // end of constructor method
	/**
	 * This method receives an reference to the Julia animation frame and gives it
	 * to a global variable
	 * @param julia - JuliaFrame
	 */
	public void heresJulia(JuliaFrame julia)
	{
		copyOfJulia = julia;
	}
	/**
	 * The paint() method invokes the method of JFrame and displays the axes if required
	 * @param g - Graphics
	 */
	public void paint(Graphics g) // When 'Window' is first
	{ 							  // shown or damaged
		super.paint(g);   
	}
	/**
	 * A method to plot a small cross on the frame to show where the animation 
	 * is taking the valur of c from
	 * @param g - Graphics2D
	 * @param x - int
	 * @param y - int
	 */
	public void plotPoint(Graphics2D g, int x, int y)
	{
		if(mouseInRange(x, y))
		{
			g.setPaint(new Color(255, 255, 255));
			g.drawLine(x-5, y, x + 5, y);
			g.drawLine(x, y - 5, x, y + 5);	
		}	
	}
	/**
	 * Redraws the background (i.e. the Mandelbrot set)
	 */
	public void background()
	{
		theAI = (BufferedImage) createImage(W + 1, H + 1);
		theAG = theAI.createGraphics();		
		reDrawMandelbrot(theAG);
	}
	/**
	 * This method redraws the Mandelbrot set by invoking the Iterator instance
	 * to return colour based on the number of iterations required for divergence
	 * or black for non-divergence. Each pixel represents a complex number which 
	 * is the value of C for the iteration of Z := Z^2 + C with the intial value
	 * of Z being zero
	 * @param g -  Graphics2D
	 */
	public void reDrawMandelbrot(Graphics2D g) // Re draw contents
	{	
		double x, y; //mathematical values of x and y positions represented by a pixel
		for(double yPixel = 0; yPixel < H; yPixel++)
		{
			for(double xPixel = 0; xPixel < W; xPixel++)
			{
				x = (minX)+(xPixel*((maxX - minX)/W));
				y = (maxY)-(yPixel*((maxY - minY)/H));		
				previousColour = currentColour;	
				currentColour = iterator.colourPicker(x, y, 0.0, 0.0);
				if(! previousColour.equals(currentColour) || xPixel == W - 1)
				{
					g.draw(new Line2D.Double(firstChangex , yPixel , xPixel , yPixel ));
					firstChangex = xPixel % (W - 1);
					g.setPaint(currentColour); 
				}				
			}
		}
	}
	/**
	 * A method to tell if the mouse is on the image of the Mandelbrot set
	 * @param mouseXp - int
	 * @param mouseYp - int
	 */
	public boolean mouseInRange(int mouseXp, int mouseYp)
	{
		return (mouseXp <= W + xAxisGap + 8 && mouseYp <= H && mouseXp >= xAxisGap + 8 && mouseYp >= 0);
	}
	/**
	 * This method drives the animation as it is invoked by the run() method
	 * of the thread in the Julia animation frame. It repaints the part of the 
	 * frame showing the cross hairs
	 * @param i - int
	 */
	public void traceC(int i)
	{	
		int prevAnimationIndex = Math.min(topArrayIndex,animationIndex);
		animationIndex = i;
		int prevAnimationX = xPixelArray[prevAnimationIndex];
		int prevAnimationY = yPixelArray[prevAnimationIndex];
		animationX = xPixelArray[animationIndex];
		animationY = yPixelArray[animationIndex];
		theRealOutput.setText(String.valueOf((double) Math.round( xArray[animationIndex]*1000000)/1000000));
		theImOutput.setText(String.valueOf((double) Math.round( yArray[animationIndex]*1000000)/1000000));
		canvas.repaint(Math.min(animationX, prevAnimationX)-5,Math.min(animationY, prevAnimationY)-5,Math.abs(animationX-prevAnimationX)+11,Math.abs(animationY-prevAnimationY)+11);
	}
	/**
	 * Turns the date and time into a string that is used in the name of the saved image
	 * @param inputDate -  Date
	 * @param format - DateFormat
	 * @return dateString -  string
	 */
	public String processDate(Date inputDate, DateFormat format)
	{
		String dateString = format.format(inputDate);
		dateString = dateString.replace(':', ' ');
		dateString = dateString.replace('/', ' ');
		return dateString;
	}	
	/**
	 * Invoked from the Julia animation frame to set the animation running
	 * or to halt the animation
	 */
	public void applicationRunning(boolean binput)
	{
		animationRunning = binput;	
		if(animationRunning) canvas.repaint();
	}
	/**
	 * Gets the text from text box holding the real component of C
	 * @return - the string holding text from text box
	 */
	public String getReCString()
	{
		return theRealOutput.getText();
	}
	/**
	 * Gets the text from text box holding the imaginary component of C
	 * @return - the string holding text from text box
	 */
	public String getImCString()
	{
		return theImOutput.getText();  
	}
	/**
	 * This subclass of JPanel will have the image of the Mandelbrot set 
	 * drawn on it.
	 */
	public class MyCanvas extends JPanel   {
		private int mouseX, mouseY; 
		private int mouseXZoom,mouseYZoom;
		private double xZoom, yZoom;
		private int xDrag, yDrag;
		private int xDragPrev = 0;
		private int yDragPrev = 0;
		private boolean dragging = false;
		private boolean mouseDown =  false, oneCross = false;
		private double xPressed, yPressed;
		public ArrayList<Integer> xPixelArrayList = new ArrayList(), yPixelArrayList = new ArrayList();
		/**
		 * The constructor will add the mouse listeners for dragging to set
		 * the animation path, and for the zoom facility
		 */
		public MyCanvas()
		{
			addMouseListener(new MouseAdapter() 			
			{
				public void mousePressed (MouseEvent e)
				{	        		           	
					repaint();
					mouseDown = true;
					mouseX = e.getX();
					mouseY = e.getY();
					if (!zoomSelect)
					{
						xPixelArrayList = new ArrayList<Integer>();
						yPixelArrayList = new ArrayList<Integer>();
						if (mouseInRange(mouseX, mouseY))
						{	
						    xPixelArrayList.add(mouseX);
						    yPixelArrayList.add(mouseY);	         
							double xValue = (minX)+((mouseX - xAxisGap - 8)*((maxX - minX)/W));
							double yValue = (maxY)-((mouseY)*((maxY - minY)/H));
							double re = Math.round(xValue*1000000);
							double im = Math.round(yValue*1000000);   
							theRealOutput.setText(String.valueOf(re/1000000));
							theImOutput.setText(String.valueOf(im/1000000)); 
						}			        
					}
					else
					{ 
						if (mouseInRange(mouseX, mouseY))
						{
							xPressed = ((minX)+((mouseX - xAxisGap - 8)*((maxX - minX)/W)));
							yPressed = ((maxY)-((mouseY)*((maxY - minY)/H)));
						}
						dragging = false;
					}};


				public void mouseReleased (MouseEvent e)
				{
					if(!zoomSelect)
					{
						if(!animationRunning && yPixelArrayList.size() > 0 && xPixelArrayList.size() > 0)
						{
							oneCross = true; // set flag to plot mouse position just once
							repaint();
						}
						if(xPixelArrayList.size() > 6)
						{
							xArray = new double[xPixelArrayList.size()];
							yArray = new double[yPixelArrayList.size()];
							xPixelArray = new int[xPixelArrayList.size()];
							yPixelArray = new int[yPixelArrayList.size()];
							topArrayIndex = Math.min(xPixelArrayList.size(),xPixelArrayList.size()) - 1;
							pathSet = true;
							for(int i = 0; i <= topArrayIndex; i++)
							{
								xPixelArray[i] = xPixelArrayList.get(i);
								yPixelArray[i] = yPixelArrayList.get(i);
								xArray[i] = ((minX)+((xPixelArray[i] - xAxisGap - 8)*((maxX - minX)/W)));
								yArray[i] = ((maxY)-((yPixelArray[i])*((maxY - minY)/H)));	
							}
							copyOfJulia.pathSet(xArray,yArray,topArrayIndex);
						}
					}
					else if(mouseInRange(e.getX(), e.getY())) // if zoomSelect and mouse on canvas
					{
						mouseXZoom = e.getX();
						mouseYZoom = e.getY();
						xZoom = (minX)+((mouseXZoom - xAxisGap - 8)*((maxX - minX)/W));
						yZoom = (maxY)-((mouseYZoom)*((maxY - minY)/H));

						minX = Math.min(xPressed, xZoom);
						maxX = Math.max(xPressed, xZoom);
						minY = Math.min(yPressed, yZoom);
						maxY = Math.max(yPressed  , yZoom);
						zoomSelect = false;
						setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
						axisWiz.setAxisLabels(axisLabel, maxY, (maxY + minY)/2, 
								minY, minX, (maxX + minX)/2, maxX);
						background();
						if (pathSet)
							for(int i = 0; i <= topArrayIndex; i++)
							{
								xPixelArray[i] = xAxisGap + 8 + (int) Math.round((xArray[i] - minX)*(W/(maxX - minX))); 
								yPixelArray[i] = (int) Math.round((maxY - yArray[i])*(H/(maxY - minY)));	

							}
						repaint();
					}
					dragging = false;   
					mouseDown = false;
				} 		    	    	
			});

			addMouseMotionListener(new MouseMotionAdapter() 
			{
				public void mouseDragged (MouseEvent e)
				{
					xDrag = e.getX();
					yDrag = e.getY();
					if (zoomSelect)
					{
						if (mouseInRange(xDrag, yDrag))
						{
							dragging = true;
							int xRepaintRange = Math.max(Math.abs(xDrag - xDragPrev),Math.max(Math.abs(mouseX - xDrag), Math.abs(mouseX - xDragPrev)));
							int yRepaintRange = Math.max(Math.abs(yDrag - yDragPrev),Math.max(Math.abs(mouseY - yDrag), Math.abs(mouseY - yDragPrev)));
							repaint(Math.min(Math.min(mouseX, xDrag),xDragPrev) - 2,  Math.min(Math.min(mouseY, yDrag),yDragPrev) - 2, xRepaintRange + 4, yRepaintRange + 4);
							xDragPrev = xDrag;
							yDragPrev = yDrag;
						}
						else
						{
							dragging = false;
							zoomSelect = false;
							setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
							repaint();
						} 
					}
					else // not zoomSelect
						if (animationRunning && mouseInRange(xDrag, yDrag))
					    {
						    JOptionPane frame = new JOptionPane();
						    JOptionPane.showMessageDialog(frame, "Animation already running, please stop the current animation before setting another path");
					    }
					    else  // not animationRunning or not mouseInRange
					    	if (mouseDown && mouseInRange(xDrag, yDrag))
					        {
						         xPixelArrayList.add(xDrag);
						         yPixelArrayList.add(yDrag);
						         double re  = (minX+((xDrag - xAxisGap - 8) *((maxX - minX)/W)));
						         double im  = (maxY-(yDrag*((maxY - minY)/H)));	
						         re = Math.round(re*1000000);
						         im = Math.round(im*1000000);   
						         theRealOutput.setText(String.valueOf(re/1000000));
						         theImOutput.setText(String.valueOf(im/1000000)); 
					         }	           
				}
			});

		} // end of constructor
		/**
		 * The paintComponent(Graphics g) method repaints the panel with the Mandelbrot 
		 * image, and it also plots the cross hairs and provides feedback when the mouse
		 * is used to select an area to zoom in  on.
		 * @param g - Graphics
		 */
		@Override
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			g.drawImage( theAI,  xAxisGap + 8,  0,  this);      
			if (animationRunning) // plot on path is animation is running
			{
				plotPoint((Graphics2D) g, xPixelArray[Math.min(animationIndex, topArrayIndex)], yPixelArray[Math.min(animationIndex, topArrayIndex)]);
			}
			if(zoomSelect && dragging) // visual feedback if zoom area is being cut out
			{
				((Graphics2D) g).setPaint(Color.WHITE);  
				g.drawRect(Math.min(mouseX, xDrag), Math.min(mouseY, yDrag), Math.abs(mouseX - xDrag), Math.abs(mouseY - yDrag));
			}
			if(showAxis) // display axes if required
			{
				axisWiz.drawAxis((Graphics2D) g, xAxisGap+8, 0, xAxisGap, yAxisGap);
			}
			if (zoomSelect) // provide visual feedback that zoom select is on
			{
				((Graphics2D) g).setPaint(new Color(255,50,50));  
				g.setFont(new Font("ARIAL", Font.BOLD, 16));
				g.drawString("Mouse drag over area to zoom in on",20, 15);
			}
			if(oneCross) 
			{
				plotPoint((Graphics2D) g, xPixelArrayList.get(xPixelArrayList.size()-1), yPixelArrayList.get(yPixelArrayList.size()-1));
				oneCross = false;
			}
			if(aCross) 
			{
				int xPix = (int) Math.round((Xpoint2 - minX)*(W/(maxX - minX))); 
			    int yPix = (int) Math.round((maxY - Ypoint2)*(H/(maxY - minY)));
				plotPoint((Graphics2D) g, xPix + xAxisGap + 8, yPix);
				aCross = false;
			}
		}
	}
	/**
	 * This method is invoked by the instance of PathInputFrame when the user clicks 
	 * OK. The data is validated before by the PathInputFrame instance before this
	 * method is invoked. Elements 0 and 2 of the parameter hold the x-coordinates
	 * of the start and end points of the path and elements 1 and 3 hold their
	 * y-coordinates.
	 * @param data - double[]
	 */
	public void recievePath(double[] data)
	{
		repaint();
		straightPath = data;
		settingPath = false;
		pathSet = true;
		double xFraction = Math.abs((data[2] - data[0])/(maxX - minX)); // horizontal range of path as fraction of plot width
		double yFraction = Math.abs((data[3] - data[1])/(maxY - minY)); // vertical range of path as fraction of plot height

		int xRangeDivisions = (int) Math.round(xFraction*(W-1)); // pixels required for horizontal range
		int yRangeDivisions = (int) Math.round(yFraction*(H-1)); // pixels required for vertical range

		int divisions = Math.max(xRangeDivisions, yRangeDivisions); // points on path will be maximum of pixels required
		topArrayIndex = divisions;

		xArray = new double[divisions + 1];
		yArray = new double[divisions + 1];

		for(int i = 0; i < divisions + 1; i++) // load mathematical points on path into arrays
		{
			xArray[i] = data[0] + i * (data[2] - data[0])/divisions;
			yArray[i] = data[1] + i * (data[3] - data[1])/divisions;
		}

		xPixelArray = new int[divisions + 1];
		yPixelArray = new int[divisions + 1];

		for(int i = 0; i <= topArrayIndex; i++) // load pixels on canvas into arrays
		{
			xPixelArray[i] = xAxisGap + 8 + (int) Math.round((xArray[i] - minX)*(W/(maxX - minX))); 
			yPixelArray[i] = (int) Math.round((maxY - yArray[i])*(H/(maxY - minY)));	

		}

		copyOfJulia.pathSet(xArray, yArray, topArrayIndex);

	}
	/**
	 * This method is invoked by the instance of PathInputFrame when the user cancels 
	 * the setting of a straight line path
	 */
	public void pathCancelled()
	{
		settingPath = false;
	}
	/**
	 * This method is invoked by the instance of PathInputFrame when the range
	 * of current values shown by the Mandelbrot plot is required for validation
	 * purposes. 
	 */
	public double[] getRange()
	{
		return new double[]{minX, maxX, minY, maxY};
	}
}