/*
*   This file is part of the Interactive Julia and Mandelbrot system.
*   The Interactive Julia and Mandelbrot system is free software: 
*	you can redistribute it and/or modify it under the terms of the 
*	GNU General Public License as published by the Free Software 
*	Foundation, either version 3 of the License, or (at your option) 
*	any later version.
*
*   The Interactive Julia and Mandelbrot system is distributed in 
*	the hope that it will be useful, but WITHOUT ANY WARRANTY; 
*	without even the implied warranty of MERCHANTABILITY or FITNESS 
*	FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
*	for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with The Interactive Julia and Mandelbrot suite.  
*	If not, see <http://www.gnu.org/licenses/>.
*/


import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import javax.imageio.ImageIO;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.border.Border;
/**
 * The OrbitFrame class is used to provide facilities for drawing a 
 * filled JJulia set and allowing the user to choose a point by mouse
 * clicking or entering coordinates, and provides a animation which
 * traces the orbit of the point. A visual record and a list of 
 * coordinates of the traced point can be saved to disk 
 *  
 */
class OrbitFrame extends JFrame {
	MandelbrotFrame mandelbrotCopy; // Global variable used to reference the JFrame holding the Mandelbrot image
	JuliaFrame juliaCopy; // Global variable used to reference the JFrame holding the Julia animation
	private static final int H = 301; // Height of window
	private static final int W = 401; // Width of window
	private Color previousColour = new Color(00, 00, 00);
	private Color currentColour = new Color(00, 00, 00);
	private Color backgroundColour = new Color(240, 240, 240); // set background of JFrame
	private double firstChangex = 0; // Used when drawing the Julia image, holds the position on line where current colour starts
	private final double minXDefault = -2; // default lower bound of x-range plotted
	private final double maxXDefault = 2;  // default upper bound of x-range plotted
	private final double minYDefault = -1.5; // default lower bound of y-range plotted
	private final double maxYDefault = 1.5; // default upper bound of y-range plotted
	private double minX = minXDefault; // lower bound of x-range plotted, may be changed by user zooming in
	private double maxX = maxXDefault; // upper bound of x-range plotted, may be changed by user zooming in
	private double minY = minYDefault; // lower bound of y-range plotted, may be changed by user zooming in
	private double maxY = maxYDefault; // upper bound of y-range plotted, may be changed by user zooming in
	private int maxIterations = 300; // maximum iterations for each complex number, if it hasn't diverged it is assumed it won't
	private int xMargin = 155; // gap between the left side of JFrame and the plotted image
	private int yMargin = 75;  // gap between lower edge of plotted image and bottom of the JFrame
	private int topOffset = 95; // gap between top of JFrame and the plotted image
	private int rightMargin = 40; // gap between right edge of plotted image and the edge of the JFrame
	private int yAxisGap = 8; // gap between y-axis and plotted image
	private int xAxisGap = 8; // gap between x-axis and plotted image
	private double imC = 0; // imaginary component of the current value of complex number c
	private double reC = 0; // real component of the current value of complex number c
	private Transaction theAL = new Transaction(); // Used to implement the buttons' actions
 	boolean animationStart = false; // true means the graphics are to be renewed just once
	private boolean traceRunning = false; // true if there is a trace running
	int delay = 240; // default delay between frames when a trace is running
	private BufferedImage theAI, theAI2, theAIToSave; // the background images with and without all trace points, and to save to disk
	private Graphics2D theAG, theAG2, theAGToSave; // the graphics for the three images
	private Color[] juliaColours = {new Color(36, 159, 120), // colours in which to plot the divergent points
			new Color(0,50,150), new Color(36, 159, 120),
			new Color(80, 180, 80), new Color(140, 50, 180), 
			new Color(130, 50, 50), new Color(0, 160, 160), 
			new Color(190, 160, 160), new Color(0, 255, 255), 
			new Color(125, 220, 140), new Color(190,220,100),
			new Color(230,230,50)};
	private int[] juliaCutOffs = {2,3,4,5,7,9,11,14, 17, 25, 50, 58};  // cut-offs for the number of iterations that trigger a change of colour
	private final JTextField theRealInput = new JTextField(); 
	private final JTextField theIminput = new JTextField();  
	private final JTextField theRealintputaOrbit = new JTextField(); 
	private final JTextField theIminputOrbit = new JTextField();  
	private JLabel label1 = new JLabel("of c =");
	private JLabel label2 = new JLabel(" + ");
	private JLabel label3 = new JLabel(" i");
	private JLabel label1Orbit = new JLabel(" + ");
	private JLabel label2Orbit = new JLabel(" i");
	private JLabel label3Orbit = new JLabel("of  z  =");
	private double[] nextPoint = new double[2];
	private Iterator iterator; // will reference an instance of Iterator which returns a colour for each pixel
	private AxisWiz axisWiz; // Will hold an instance for drawing the axes and setting the axis labels' values
	private JMenuBar menubar = new JMenuBar();
	private JMenu file;
	private JMenu zoom, axes, trace;
	private JMenuItem save, exit, setRanges, normal, show, hide, showCurrent, showAll, saveTraceData;
	private boolean showAllPoints1 = false; // true if alll points in a trace are shown
	private boolean showAxis = false; // true if the axes are visible
	private boolean firstDisplay = true; // true if the first Julia set has been plotted
	private JLabel[] axisLabel = {new JLabel(String.valueOf(maxY)), new JLabel(String.valueOf((maxY + minY)/2)), 
			new JLabel(String.valueOf(minY)), new JLabel(String.valueOf(minX)),
			new JLabel(String.valueOf((maxX + minX)/2)), new JLabel(String.valueOf(maxX))};  
	String outputC; // string with value of C for saving to disk
	private boolean zoomSelect = false; // true if the user has opted to select an area to zoom in on with the mouse
	private int oldXPixel = 0, oldYPixel = 0, newXPixel = 0, newYPixel = 0; // old and new coordinates of point iterated during the trace
	JOptionPane frame = new JOptionPane();
	BufferedWriter textOut; // used to save coordinates of traced points
	JButton a; // restart button
	JButton h; // trace orbit button
	DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss"); // format for putting date in name of saved image
	boolean fromAnimation = false; // flag to say whether the value of c should be got from the Mandelbrot or Julia frame
	ArrayList<double[]> traceData = new ArrayList<double[]>(); // holds coordinates of traced points so they can be saved to disk
	MyJOCanvas canvas = new MyJOCanvas(); // A subclass of JPanel on which the graphic image is drawn

    /**
     * The constructor adds the menu bar and drop down menus, and sets
	 * the location, size, etc of the frame. It creates the instances of Iterator 
	 * and Axiswiz that will be used by this instance of OrbitFrame.The constructor
	 * also receives as parameters the instances of MandelbrotFrame and JuliaFrame 
	 * with which this frame will interact and gives them to the global variables
	 * mandelbrotCopy. and juliaCopy. It adds buttons, JLabels and text boxes.
	 * It also creates the instance of Animate which creates the thread which runs the 
	 * animation
     * @param mandelbrot - MandelbrotFrame
     * @param julia - JuliaFrame
     */
	public OrbitFrame(MandelbrotFrame mandelbrot, JuliaFrame julia) {
		
		mandelbrotCopy = mandelbrot;	
		juliaCopy = julia;
		setLocation(750, 398);
		setSize(W + xMargin + rightMargin, H + yMargin + topOffset ); // Size of drawing area
		setDefaultCloseOperation(EXIT_ON_CLOSE); 
		setResizable(false);	
		setTitle("Julia set orbit plotter");
		Animate animation = new Animate();
		Container cp = getContentPane();
		cp.setLayout(null);
		cp.setBackground(backgroundColour);

		a = new JButton("Restart");
		a.setBounds(10, 1 * 50 + 40  , 95, 40);
		a.addActionListener(theAL);
		cp.add(a);
		a.setEnabled(false);

		JButton b = new JButton("Stop");
		b.setBounds(10, 2 * 50 + 40 , 95, 40);
		b.addActionListener(theAL);
		cp.add(b);

		JButton e = new JButton("Faster");
		e.setBounds(10, 3 * 50 + 70, 95, 40);
		e.addActionListener(theAL);
		cp.add(e);

		JButton f = new JButton("Slower");
		f.setBounds(10, 4 * 50 + 70, 95, 40);
		f.addActionListener(theAL);
		cp.add(f);

		JButton g = new JButton("Draw Julia set");
		g.setBounds(5, topOffset - 88 , 115, 35);
		g.addActionListener(theAL);
		cp.add(g);

		h = new JButton("Trace Orbit");
		h.setBounds(10, topOffset + H + yMargin - 100 , 110, 40);
		h.addActionListener(theAL);
		cp.add(h);
		h.setEnabled(false);

		JButton i = new JButton("Get C");
		i.setBounds(xMargin + 250, topOffset - 88, 65, 30);
		i.addActionListener(theAL);
		cp.add(i);
		
		canvas.setBounds(xMargin - xAxisGap - 8, topOffset - 48, W + xAxisGap + 9, H + yAxisGap + 8 );
		canvas.setBackground(backgroundColour);
		cp.add(canvas);

		theRealInput.setBounds(xMargin + 30,topOffset - 85,80,30);
		cp.add(theRealInput);	 

		theIminput.setBounds(xMargin + 138,topOffset - 85,80,30);
		cp.add(theIminput);

		label1.setBounds(xMargin - 25,topOffset - 85,55,30);
		label1.setFont(new Font("Serif", Font.PLAIN, 20));
		cp.add(label1);

		label2.setBounds(xMargin + 113,topOffset - 85,55,30);
		label2.setFont(new Font("Serif", Font.PLAIN, 20));
		cp.add(label2);

		label3.setBounds(xMargin + 217,topOffset - 85,40,30);
		label3.setFont(new Font("Serif", Font.ITALIC, 20));
		cp.add(label3);

		theRealintputaOrbit.setBounds(xMargin + 50,topOffset + H + yMargin - 87,80,30);
		cp.add(theRealintputaOrbit);	 

		theIminputOrbit.setBounds(xMargin + 170,topOffset + H + yMargin - 87,80,30);
		cp.add(theIminputOrbit);

		label1Orbit.setBounds(xMargin + 140,topOffset + H + yMargin - 87 ,50,30);
		label1Orbit.setFont(new Font("Serif", Font.PLAIN, 20));
		cp.add(label1Orbit);

		label2Orbit.setBounds(xMargin + 250,topOffset + H + yMargin - 88,50,30);
		label2Orbit.setFont(new Font("Serif", Font.ITALIC, 20));
		cp.add(label2Orbit);

		label3Orbit.setBounds(xMargin - 20,topOffset + H + yMargin - 88,100,30);
		label3Orbit.setFont(new Font("Serif", Font.PLAIN, 20));
		cp.add(label3Orbit);		 

		iterator = new Iterator(maxIterations, juliaColours, juliaCutOffs);
		axisWiz = new AxisWiz(W, H);

		axisWiz.placeAxisLables(this, axisLabel, xMargin, topOffset);

		axisWiz.setAxisLabels(axisLabel, maxY, (maxY + minY)/2, 
				minY, minX, (maxX + minX)/2, maxX);

		menubar = new JMenuBar();
		setJMenuBar(menubar);

		file = new JMenu("File");
		zoom = new JMenu("Zoom");
		axes = new JMenu("Axes");
		trace = new JMenu("Trace");
		
		file.setEnabled(false);
		zoom.setEnabled(false);
		trace.setEnabled(false);

		menubar.add(file);
		menubar.add(zoom);
		menubar.add(axes);
		menubar.add(trace);

		save = new JMenuItem("Save Image");
		exit = new JMenuItem("Exit");
		setRanges = new JMenuItem("Select Area With Mouse (on/off)");
		normal = new JMenuItem("Normal");
		show = new JMenuItem("Show");
		hide = new JMenuItem("Hide");
		showCurrent = new JMenuItem("Show Current Points");
		showAll = new JMenuItem("Show All Points");
		saveTraceData = new JMenuItem("Save Trace Data");

		file.add(save);
		file.add(saveTraceData);
		file.add(exit);
		zoom.add(setRanges);
		zoom.add(normal);
		axes.add(show);
		axes.add(hide);
		trace.add(showCurrent);
		trace.add(showAll);

		//Radio Buttons
		JRadioButton mandelbrotGet = new JRadioButton("from mandelbrot");
		mandelbrotGet.setSelected(true);

		JRadioButton animationGet = new JRadioButton("from animation");
		animationGet.setSelected(true);

		ButtonGroup group = new ButtonGroup();
		group.add(mandelbrotGet);
		group.add(animationGet);

		mandelbrotGet.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				fromAnimation = false;
			}
		});

		animationGet.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				fromAnimation = true;
			}
		});

		mandelbrotGet.setBounds(xMargin + 315, topOffset - 92, 130, 20);
		animationGet.setBounds(xMargin + 315, topOffset - 75, 130, 20);

		cp.add(mandelbrotGet);
		cp.add(animationGet);

		exit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				System.exit(0);
			}
		});

		save.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {

				// copy visual information to the buffered image for saving

				theAIToSave = (BufferedImage) createImage(W + 130, H + 100);
				theAGToSave  = theAIToSave.createGraphics();
				theAGToSave.drawImage(theAI2, 80, 50, null);
				outputC = "C = " + String.valueOf(reC) + " + " + String.valueOf(imC) + "i";	
				theAGToSave.drawString(outputC, 10,  20);
				if(showAxis) 
				{
					axisWiz.drawAxis(theAGToSave, 80, 50, 8, 8);
					theAGToSave.drawString(axisWiz.axisLabelNumber(maxY), 0, 53);
					theAGToSave.drawString(axisWiz.axisLabelNumber((minY + maxY)/2),0, 53 + H/2);
					theAGToSave.drawString(axisWiz.axisLabelNumber(maxY),0, 53 + H);
					theAGToSave.drawString(axisWiz.axisLabelNumber(minX), 47, 50 + H + 32);
					theAGToSave.drawString(axisWiz.axisLabelNumber((minX + maxX)/2), 47 + W/2, 50 + H + 32);
					theAGToSave.drawString(axisWiz.axisLabelNumber(maxX), 47 + W, 50 + H + 32);
				}


				// save buffer showing trace points (if any)
				try {
					ImageIO.write(theAIToSave, "png", new File("orbit " + processDate(new Date(), dateFormat) + ".png"));
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		});

		saveTraceData.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// open new file for subsequent trace data
				try {
					textOut = new BufferedWriter(new FileWriter("trace" + processDate(new Date(), dateFormat) + ".txt"));
					textOut.write("Draw: c  = " + reC + " + " + imC +  " i " + new Date());
					textOut.newLine();
					for(int i = 0; i < traceData.size(); i++)
					{
						textOut.write("Z = " + traceData.get(i)[0] + " " + traceData.get(i)[1] + " i " );
						textOut.newLine();
					}
					textOut.close();
				} catch (IOException e1) {
					e1.printStackTrace();
				}

			}
		});

		showAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				showAllPoints1 = true;
				canvas.repaint  ();
			}
		});

		showCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				showAllPoints1 = false;
				canvas.repaint();
			}
		});

		show.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				showAxis = true;
				axisWiz.setVisibility(axisLabel, true);
				repaint();
			}
		});

		hide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				showAxis = false;
				axisWiz.setVisibility(axisLabel, false);
				repaint();
			}
		});  	  

		normal.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				minX = minXDefault;
				maxX = maxXDefault;
				minY = minYDefault;
				maxY = maxYDefault;
				axisWiz.setAxisLabels(axisLabel, maxY, (maxY + minY)/2, 
						minY, minX, (maxX + minX)/2, maxX);
				background();
				animationStart = true;
				repaint();
			}
		}); 

		setRanges.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (!firstDisplay) 
				{
					zoomSelect = !zoomSelect; // toggle
					if(zoomSelect)
					{
						canvas.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
					}
					else
					{
						canvas.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					}
					repaint();
				}
			}
		});
		
		setVisible(true);	 		
	} // end of constructor

   /**
     * 
	 * The class transaction provides the action listener for the buttons
	 *
	 */
	class Transaction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			String actionIs = ae.getActionCommand();
			if (actionIs.equals("Restart"))
			{ 
				traceRunning = true;
			} 
			if (actionIs.equals("Stop"))
			{  
				traceRunning = false;
			}
			if (actionIs.equals("Faster"))
			{
				if (delay > 40)
					delay -= 10;
			}
			if (actionIs.equals("Slower"))
			{
				delay += 10;
			}	 
			if (actionIs.equals("Get C"))
			{
				if(!fromAnimation)
				{
					theRealInput.setText(mandelbrotCopy.getReCString());
					theIminput.setText(mandelbrotCopy.getImCString());	 
				}else
				{
					double re = Math.round(juliaCopy.getRe()*1000000);
					double im = Math.round(juliaCopy.getIm()*1000000);
					theRealInput.setText(Double.toString(re/1000000));
					theIminput.setText(Double.toString(im/1000000));	 
				}
			}	 
			if (actionIs.equals("Draw Julia set"))
			{
				boolean validInput = true;
				if (theRealInput.getText().length() ==  0 && theIminput.getText().length()  == 0)
				{
					theRealInput.setText(mandelbrotCopy.getReCString());
					theIminput.setText(mandelbrotCopy.getImCString());	    		
				}
				if (theRealInput.getText().length() ==  0) theRealInput.setText("0");
				if (theIminput.getText().length() ==  0) theIminput.setText("0");    		    	  

				try {
					reC = Double.parseDouble(theRealInput.getText());
					imC = Double.parseDouble(theIminput.getText());
				} catch (NumberFormatException e) {
					JOptionPane.showMessageDialog(frame, "Please Enter A Valid Number");
					validInput = false;
				}
				if (validInput)
				{
					traceData.clear();
					animationStart = true;
					firstDisplay = false;
					traceRunning = false;
					background();
					h.setEnabled(true);
					file.setEnabled(true);
					zoom.setEnabled(true);
					trace.setEnabled(true);
					repaint(xMargin, topOffset, W, H);	
				}
			}	
			if (actionIs.equals("Trace Orbit"))
			{
				boolean validInput = true;
				try {
					nextPoint[0] = Double.parseDouble(theRealintputaOrbit.getText());
					nextPoint[1] = Double.parseDouble(theIminputOrbit.getText());
				} catch (NumberFormatException e) {
					JOptionPane.showMessageDialog(frame, "Please Enter A Valid Number");
					validInput = false;
				}
				if (validInput)
				{
					traceData.clear();
					traceData.add(nextPoint);
					background();
					a.setEnabled(true);
					newXPixel = xAxisGap + 8 + (int) Math.round(((nextPoint[0] - minX)*(W/(maxX - minX))));
					newYPixel = (int) Math.round(((maxY - nextPoint[1])*(H/(maxY - minY))));
					canvas.repaint();
					traceRunning = true;
				}
			}	
		}
	}
	/**
	 * 
	 * The class Animate provides the thread that runs the animation
	 *
	 */
	class Animate implements Runnable
	{

		private Thread theAnimation;
		public Animate()
		{
			theAnimation = new Thread(this);
			theAnimation.start();
		}
        /**
         * The run method moves the trace onto the next point if a trace is running,
         * repaints the screen just once if animationStart has been set to true, and 
         * repaints regularly during a user selection of an area to zoom in on with 
         * the mouse. The thread sleeps for a short time set by the variable delay.
         */
		public void run() {
			while( true )
			{
				try
				{				
					if (animationStart || zoomSelect )
					{
						canvas.repaint();		
						animationStart = false;
					}
					if (traceRunning)
					{
						oldXPixel = newXPixel;
						oldYPixel = newYPixel;
						nextPoint = nextPoint(reC,imC,nextPoint[0],nextPoint[1]);
						newXPixel = xAxisGap + 8 + (int) Math.round(((nextPoint[0] - minX)*(W/(maxX - minX))));
						newYPixel = (int) Math.round(((maxY - nextPoint[1])*(H/(maxY - minY))));
						canvas.renewPoints(oldXPixel,oldYPixel,newXPixel,newYPixel);
					}

					Thread.sleep(delay);				
				}
				catch (InterruptedException e) {}
			}	
		}	
	}

	public void update(Graphics g)
	{
		paint((Graphics2D) g);
	}
	/**
     * The paint() method invokes the method of JFrame and displays the axes if required. 
     * A black rectangle is displayed if no Julia set has yet been plotted. 
     * @param g - Graphics
     */
	public void paint(Graphics g) // When 'Window' is first
	{ // shown or damaged
		super.paint(g);
	}
	/**
	 * This method redraws the Julia set by invoking the Iterator instance
	 * to return colour based on the number of iterations required for divergence
	 * or black for non-divergence. Each pixel represents a complex number which is
	 * the initial value of Z for the iteration of Z := Z^2 + C.
	 * @param g -  Graphics2D
	 */
	public void reDrawJulia(Graphics2D g) // Re draw contents  
	{
		double x, y;
		for(double yPixel = 0; yPixel < H; yPixel++)
		{
			for(double xPixel = 0; xPixel < W; xPixel++)
			{
				x = (minX)+((xPixel)*((maxX - minX)/W));
				y = (maxY)-(yPixel*((maxY - minY)/H));
				previousColour = currentColour;		
				currentColour = iterator.colourPicker(reC, imC, x, y);
				if(! previousColour.equals(currentColour) || xPixel == W - 1)
				{
					g.draw(new Line2D.Double(firstChangex, yPixel, xPixel, yPixel));
					firstChangex = xPixel % (W - 1);
					g.setPaint(currentColour);
				}				
			}
		}
	}
    /**
     * Uses the coordinates of hte current point in the trace to calculate the 
     * coordinates of the next point.
     * @param x - double
     * @param y -  double
     * @param a -  double
     * @param b -  double
     * @return - array of int
     */
	public double[] nextPoint(double x, double y, double a, double b)
	{
		double[] array = iterator.oneIteration(x, y, a, b);
		traceData.add(array);
		if (array[0] > maxXDefault || array[0] < minXDefault ||array[1] > maxYDefault || array[1] < minYDefault) traceRunning = false; // stop the animation if the point goes off the full-sized Julia plot
		return array;	// return the new point	
	}
    /**
     * Draws the Julia set on two BufferedImage instances, one will have all 
     * trace points plotted on it
     */
	public void background() 
	{  // re-plots the Julia set on both alternate images
		theAI = (BufferedImage) createImage(W, H);
		theAI2 = (BufferedImage) createImage(W, H);
		theAG = theAI.createGraphics();	
		theAG2 = theAI2.createGraphics();	
		reDrawJulia(theAG);
		reDrawJulia(theAG2);
		theAG2.setPaint(new Color(255, 255, 255)); // record the point as a single pixel on the second alternate image
		if(traceData.size() != 0)
		{
			for(int i = 0; i < traceData.size(); i++)
			{
				if(traceData.get(i)[0] >= minX && traceData.get(i)[0] <= maxX && traceData.get(i)[1] >= minY && traceData.get(i)[1] <= maxY)
				{
					int nextXPixel = (int) Math.round((traceData.get(i)[0] - minX)*(W/(maxX - minX)));
					int nextYPixel = (int) Math.round(((maxY - traceData.get(i)[1])*(H/(maxY - minY))));
					theAG2.drawLine(nextXPixel, nextYPixel, nextXPixel, nextYPixel);
				}
			}
		}
	}
    /**
     * Turns the date and time into a string that is used in the name of the saved image
     * @param inputDate - Date
     * @param format - DateFormat
     * 
     */
	public String processDate(Date inputDate, DateFormat format) 
	{ // formats date and time for output in text file
		String dateString = format.format(inputDate);
		dateString = dateString.replace(':', ' ');
		dateString = dateString.replace('/', ' ');
		return dateString;
	}
	/**
	 * This subclass of JPanel will have the images of the Julia sets 
     * drawn on it
	 */
	public class MyJOCanvas extends JPanel   {
		private int mouseX, mouseY;
		private int mouseXZoom, mouseYZoom;
		private double xZoom, yZoom;
		private double x, y; // The mathematical x and y values corresponding to image pixels
		private int xDrag, yDrag;
		private boolean dragging = false;
		/**
         * The constructor adds the mouse listeners for zooming in on a selected part
         * of the plot, or selecting a point for the trace
         */
		public MyJOCanvas()
		{
			addMouseListener(new MouseAdapter() 		
			{
				public void mousePressed (MouseEvent e)
				{	    

					mouseX = e.getX(); // remember position of mouse
					mouseY = e.getY();
					if (mouseInRange(mouseX, mouseY)) // if on canvas calculate coordinates
					{
						x = (minX)+((mouseX - xAxisGap - 8)*((maxX - minX)/W));
						y = (maxY)-(mouseY*((maxY - minY)/H));

						if(!zoomSelect) // if user is not zooming put coordinates 		          
						{	            // rounded to 6 d.p. in text boxes        
							double re = Math.round(x*1000000);
							double im = Math.round(y*1000000);   
							theRealintputaOrbit.setText(String.valueOf(re/1000000));
							theIminputOrbit.setText(String.valueOf(im/1000000)); 
						}
					}
					dragging = false;
				}
				public void mouseReleased (MouseEvent e)
				{
					if(zoomSelect && mouseInRange(e.getX(), e.getY()))
					{ // if on canvas, zooming and an area is selected
						mouseXZoom = e.getX();
						mouseYZoom = e.getY(); // remember positions
						xZoom = (minX)+((mouseXZoom - xAxisGap - 8)*((maxX - minX)/W)); // calculate coordinates
						yZoom = (maxY)-(mouseYZoom*((maxY - minY)/H));

						minX = Math.min(x, xZoom); // work out new zoomed in ranges
						maxX = Math.max(x, xZoom);
						minY = Math.min(y, yZoom);
						maxY = Math.max(y, yZoom);

						axisWiz.setAxisLabels(axisLabel, maxY, (maxY + minY)/2, 
								minY, minX, (maxX + minX)/2, maxX); // redraw axes
						background(); // replot the Julia set within new ranges
					}
					zoomSelect = false; // no longer zooming in
					repaint(); // redisplay whole frame
					setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					dragging = false;   // finished dragging	
				}
			});

			addMouseMotionListener(new MouseMotionAdapter() 
			{
				public void mouseDragged (MouseEvent e)
				{
					xDrag = e.getX(); // remember position
					yDrag = e.getY();
					if (mouseInRange(xDrag, yDrag))
					{
						dragging = true; // if on canvas set dragging to true
					}
					else // if not on canvas come out of zoom mode
					{
						dragging = false;
						zoomSelect = false;
						setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					} 
					repaint();
				}
			});
		}
		/**
		 * The paintComponent (Graphics g) method repaints the panel with one or other
		 * of the Julia images according to whether all points in the trace are to be 
		 * shown or not. It provides feedback when the mouse is used to select an area 
		 * to zoom in on.
		 */
		@Override
		public void paintComponent(Graphics g) {
            super.paintComponent(g);
            if(firstDisplay)
    		{
    			((Graphics2D) g).setPaint(Color.BLACK);
    			((Graphics2D) g).fill( new Rectangle2D.Double( xAxisGap + 8, 0, W, H) );	
    		}
            else if(showAllPoints1 == false)
    		{
    			g.drawImage( theAI,  xAxisGap + 8,  0,  this); // current point only shown
    		}else
    		{
    			g.drawImage( theAI2,  xAxisGap + 8,  0,  this); // all trace points shown
    		}

    		if(traceRunning) // plot point if trace running
    		{
    			plotPoint((Graphics2D) g, newXPixel, newYPixel);
    		}  
    		 if (zoomSelect) // visual feedback that zoom select is on
 			{
 				((Graphics2D) g).setPaint(new Color(255,50,50));  
 				g.setFont(new Font("ARIAL", Font.BOLD, 16));
 				g.drawString("Mouse drag over area to zoom in on",20, 15);
 			}
            if(zoomSelect && dragging) // provide visual feedback if user is selecting zoom area
    		{
    			((Graphics2D) g).setPaint(Color.WHITE);
    			g.drawRect(Math.min(mouseX, xDrag), Math.min(mouseY, yDrag), Math.abs(mouseX - xDrag), Math.abs(mouseY - yDrag));
    		}
            if (showAxis)
    		{
    			axisWiz.drawAxis((Graphics2D) g, xAxisGap + 8, 0, xAxisGap, yAxisGap);
    		}
        }
		/**
		 * A method to tell if the mouse is on the image of the Mandelbrot set
		 * @param mouseXp - int
		 * @param mouseYp - int
		 */
		public boolean mouseInRange(int mouseXp, int mouseYp)
		{
			return (mouseXp <= W + xAxisGap + 8 && mouseYp <= H && mouseXp >= xAxisGap + 8 && mouseYp >= 0);
		}
		/**
		 * A method to repaint only the necessary part of the screen as a new iterate is plotted
		 * @param oldXixel - int
		 * @param newXPixel - int
		 * @param oldYixel - int
		 * @param newYPixel - int
		 */
		public void renewPoints(int oldXPixel,int oldYPixel,int newXPixel,int newYPixel)
		{
			repaint(Math.min(oldXPixel, newXPixel)-3,Math.min(oldYPixel, newYPixel)-3,Math.abs(newXPixel-oldXPixel)+7,Math.abs(newYPixel-oldYPixel)+7);
		}
		/**
		 * Method to plot a point on the image of a Julia set. It is used to plot
		 * the iterates of the point being traced.
		 * @param g - Graphics2D
		 * @param xPixel - int
		 * @param yPixel - int
		 */
		public void plotPoint(Graphics2D g, int xPixel, int yPixel)
		{
			g.setPaint(new Color(255, 255, 255)); // plot current iterate small white cross on the panel
			g.drawLine(xPixel-3, yPixel, xPixel + 3, yPixel);
			g.drawLine(xPixel, yPixel - 3, xPixel, yPixel + 3);	
			
			theAG2.setPaint(new Color(255, 255, 255)); // record the point as a single pixel on the second alternate image
			theAG2.drawLine(xPixel - xAxisGap - 8, yPixel, xPixel - xAxisGap - 8, yPixel);
		}
	}
	
}