/*
*   This file is part of the Interactive Julia and Mandelbrot system.
*   The Interactive Julia and Mandelbrot system is free software: 
*	you can redistribute it and/or modify it under the terms of the 
*	GNU General Public License as published by the Free Software 
*	Foundation, either version 3 of the License, or (at your option) 
*	any later version.
*
*   The Interactive Julia and Mandelbrot system is distributed in 
*	the hope that it will be useful, but WITHOUT ANY WARRANTY; 
*	without even the implied warranty of MERCHANTABILITY or FITNESS 
*	FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
*	for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with The Interactive Julia and Mandelbrot suite.  
*	If not, see <http://www.gnu.org/licenses/>.
*/

import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
/**
 * An instance of this class is used when the user selects the option 
 * to set a straight line path across the plot of the Mandelbrot set
 */
public class PathInputFrame extends JFrame {
	int W = 300;
	int H = 250;
	JTextField [] input = new JTextField[4]; 
	JLabel from, to, plus1, plus2, i1, i2;
	JButton okay = new JButton("okay");
	JButton cancel = new JButton("cancel");
	MandelbrotFrame m;
	private Transaction theAL = new Transaction();
	boolean allValid;
	double[] range;
	/**
	 * The constructor receives the instance of MandelbrotFrame in order to send back
	 * information, and adds four text JTextFields for the user to input coordinates 
	 * of two points and adds the OK and cancel buttons to the frame.
	 */
	public PathInputFrame(MandelbrotFrame mframe)
	{
		m = mframe;
		this.setTitle("Straight line path");

		Container cp = getContentPane();
		cp.setLayout(null);

		setSize(W, H);
		setVisible(true);

		input[0] = new JTextField();
		input[1] = new JTextField();
		input[2] = new JTextField();
		input[3] = new JTextField();

		input[0].setBounds(80, 60, 50, 30);
		input[1].setBounds(200, 60, 50, 30);
		input[2].setBounds(80, 120, 50, 30);
		input[3].setBounds(200, 120, 50, 30);

		cp.add(input[0]);
		cp.add(input[1]);
		cp.add(input[2]);
		cp.add(input[3]);

		from = new JLabel("from:");
		to = new JLabel("to:");

		plus1 = new JLabel("+");
		plus2 = new JLabel("+");

		i1 = new JLabel("i");
		i2 = new JLabel("i");
		
		JLabel ins = new JLabel("<html>Your path must lie within the visible<br> part of the Mandelbrot image</html>");
		ins.setBounds(15, 0, 280, 50);
		cp.add(ins);

		from.setBounds(10, 50, 50, 30);
		to.setBounds(10, 120, 50, 30);

		plus1.setBounds(160, 60, 50, 30);
		plus2.setBounds(160, 120, 50, 30);

		i1.setBounds(260, 60, 50, 30);
		i2.setBounds(260, 120, 50, 30);

		from.setFont(new Font("ARIAL", Font.PLAIN, 16));
		to.setFont(new Font("ARIAL", Font.PLAIN, 16));
		plus1.setFont(new Font("ARIAL", Font.PLAIN, 16));
		plus2.setFont(new Font("ARIAL", Font.PLAIN, 16));
		i1.setFont(new Font("ARIAL", Font.ITALIC, 16));
		i2.setFont(new Font("ARIAL", Font.ITALIC, 16));
		ins.setFont(new Font("ARIAL", Font.ITALIC, 14));

		cp.add(from);
		cp.add(to);
		cp.add(plus1);
		cp.add(plus2);
		cp.add(i1);
		cp.add(i2);

		okay.setBounds(70, 170, 80, 30);
		okay.addActionListener(theAL);

		cancel.setBounds(180, 170, 80, 30);
		cancel.addActionListener(theAL);

		cp.add(okay);
		cp.add(cancel);
		// Add DocumentListeners to JTextFieds for immediate validation of input.
		// When there is a document event the Document instance is passed to the
		// clear method
		for(int i = 0; i < input.length; i++)
		{
			input[i].getDocument().addDocumentListener(new DocumentListener() {
				  public void changedUpdate(DocumentEvent e) {
				    clear(e.getDocument());
				  }
				  public void removeUpdate(DocumentEvent e) {
				    clear(e.getDocument());
				  }
				  public void insertUpdate(DocumentEvent e) {
				    clear(e.getDocument());
				  }
				});
		}	
		// The window listener is to ensure closing the frame is treated the
		// way as if the user pressed the Cancel button
		 this.addWindowListener(new WindowAdapter() {
	            public void windowClosing(WindowEvent ev) {
	               cancel.doClick();
	            }
	     });

	} // end of constructor

	class Transaction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			String actionIs = ae.getActionCommand();

			if (actionIs.equals("okay"))
			{ 
				range = m.getRange(); // ask for current range of Mandelbrot plot
				double[] data = new double[4];
				allValid = true;
				boolean currentValid = true;
				for(int i = 0; i < input.length; i++) // data validation loop
				{
					try
					{
						data[i] = Double.parseDouble(input[i].getText());
					}
					catch(NumberFormatException e)
					{
						allValid = false; // data is not valid if converting to a number raises an exception
						input[i].setBackground(new Color(255,240,240));
					}
					// validate that the path is contained within the Mandelbrot plot
					currentValid = data[i] >= range[(i%2)*2] && data[i] <= range[((i%2)*2)+1];	// (i%2)*2 is 0 for x coordinates and 2 for y coordinates	
					if(!currentValid) 
					{
						input[i].setBackground(new Color(255,240,240));
						allValid = false; 
					}
				}
	
				if(allValid) // if coordinates are valid send path data to MandelbrotFrame instance
				{
					m.recievePath(data);
					dispose(); // dispose of self after fulfilling purpose
				}
			}

			if(actionIs.equals("cancel"))
			{
				m.pathCancelled();
				dispose(); // dispose of self if the user cancels
			}
		}
	}
	/**
	 * The clear method finds out which JTextField has been typed into and validates 
	 * the new contents. It checks whether a valid number is present within the range
	 * of the Mandelbrot plot.
	 * @param x - Document
	 */
	public void clear(Document x) 
	{	
		int a = -1;    
		for(int i = 0; i < input.length; i++) // Find out which JTextField has changed
		{
			if(input[i].getDocument() == x)
			{
				a = i;
			}
		}
		double temp;
		boolean valid = true; 
		input[a].setBackground(new Color(255,255,255));
		range = m.getRange(); // // ask for current range of Mandelbrot plot
		try
		{
			temp = Double.parseDouble(input[a].getText());
			valid = temp >= range[(a%2)*2] && temp <= range[((a%2)*2)+1]; // check that number is in range
			if(!valid) input[a].setBackground(new Color(255,240,240)); 
		}
		catch(NumberFormatException e)
		{
			input[a].setBackground(new Color(255,240,240));
		}
	}
}
